#
# Copyright (C) 2006-2010  Nexedi SA
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

import neo.lib

from neo.storage.handlers import BaseMasterHandler
from neo.lib import protocol

class InitializationHandler(BaseMasterHandler):

    def answerNodeInformation(self, conn):
        self.app.has_node_information = True

    def notifyNodeInformation(self, conn, node_list):
        # the whole node list is received here
        BaseMasterHandler.notifyNodeInformation(self, conn, node_list)

    def answerPartitionTable(self, conn, ptid, row_list):
        app = self.app
        pt = app.pt
        pt.load(ptid, row_list, self.app.nm)
        if not pt.filled():
            raise protocol.ProtocolError('Partial partition table received')
        neo.lib.logging.debug('Got the partition table :')
        self.app.pt.log()
        # Install the partition table into the database for persistency.
        cell_list = []
        num_partitions = app.pt.getPartitions()
        unassigned_set = set(xrange(num_partitions))
        for offset in xrange(num_partitions):
            for cell in pt.getCellList(offset):
                cell_list.append((offset, cell.getUUID(), cell.getState()))
                if cell.getUUID() == app.uuid:
                    unassigned_set.remove(offset)
        # delete objects database
        if unassigned_set:
            neo.lib.logging.debug(
                            'drop data for partitions %r' % unassigned_set)
            app.dm.dropPartitions(num_partitions, unassigned_set)

        app.dm.setPartitionTable(ptid, cell_list)
        self.app.has_partition_table = True

    def answerLastIDs(self, conn, loid, ltid, lptid):
        self.app.dm.setLastOID(loid)
        self.app.has_last_ids = True

    def notifyPartitionChanges(self, conn, ptid, cell_list):
        # XXX: This is safe to ignore those notifications because all of the
        # following applies:
        # - we first ask for node information, and *then* partition
        #   table content, so it is possible to get notifyPartitionChanges
        #   packets in between (or even before asking for node information).
        # - this handler will be changed after receiving answerPartitionTable
        #   and before handling the next packet
        neo.lib.logging.debug('ignoring notifyPartitionChanges during '\
            'initialization')



#
# Copyright (C) 2009-2010  Nexedi SA
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

import unittest
from mock import Mock
from neo.lib import protocol
from neo.tests import NeoUnitTestBase
from neo.lib.protocol import Packet, NodeTypes, NodeStates
from neo.master.handlers.election import ClientElectionHandler, \
        ServerElectionHandler
from neo.master.app import Application
from neo.lib.exception import ElectionFailure
from neo.lib.connection import ClientConnection

# patch connection so that we can register _addPacket messages
# in mock object
def _addPacket(self, packet):
    if self.connector is not None:
        self.connector._addPacket(packet)

class MasterClientElectionTests(NeoUnitTestBase):

    def setUp(self):
        NeoUnitTestBase.setUp(self)
        # create an application object
        config = self.getMasterConfiguration(master_number=1)
        self.app = Application(config)
        self.app.pt.clear()
        self.app.em = Mock()
        self.app.uuid = self._makeUUID('M')
        self.app.server = (self.local_ip, 10000)
        self.app.name = 'NEOCLUSTER'
        self.election = ClientElectionHandler(self.app)
        self.app.unconnected_master_node_set = set()
        self.app.negotiating_master_node_set = set()
        for node in self.app.nm.getMasterList():
            self.app.unconnected_master_node_set.add(node.getAddress())
        # define some variable to simulate client and storage node
        self.storage_port = 10021
        self.master_port = 10011
        # apply monkey patches
        self._addPacket = ClientConnection._addPacket
        ClientConnection._addPacket = _addPacket

    def tearDown(self):
        # restore patched methods
        ClientConnection._addPacket = self._addPacket
        NeoUnitTestBase.tearDown(self)

    def identifyToMasterNode(self):
        node = self.app.nm.getMasterList()[0]
        node.setUUID(self.getNewUUID())
        conn = self.getFakeConnection(uuid=node.getUUID(),
                address=node.getAddress())
        return (node, conn)

    def _checkUnconnected(self, node):
        addr = node.getAddress()
        self.assertTrue(addr in self.app.unconnected_master_node_set)
        self.assertFalse(addr in self.app.negotiating_master_node_set)

    def _checkNegociating(self, node):
        addr = node.getAddress()
        self.assertTrue(addr in self.app.negotiating_master_node_set)
        self.assertFalse(addr in self.app.unconnected_master_node_set)

    def test_connectionStarted(self):
        node, conn = self.identifyToMasterNode()
        self.assertTrue(node.isUnknown())
        self._checkUnconnected(node)
        self.election.connectionStarted(conn)
        self.assertTrue(node.isUnknown())
        self._checkNegociating(node)

    def test_connectionFailed(self):
        node, conn = self.identifyToMasterNode()
        self.assertTrue(node.isUnknown())
        self._checkUnconnected(node)
        self.election.connectionFailed(conn)
        self._checkUnconnected(node)
        self.assertTrue(node.isUnknown())

    def test_connectionCompleted(self):
        node, conn = self.identifyToMasterNode()
        self.assertTrue(node.isUnknown())
        self._checkUnconnected(node)
        self.election.connectionCompleted(conn)
        self._checkUnconnected(node)
        self.assertTrue(node.isUnknown())
        self.checkAskPrimary(conn)

    def _setNegociating(self, node):
        self._checkUnconnected(node)
        addr = node.getAddress()
        self.app.negotiating_master_node_set.add(addr)
        self.app.unconnected_master_node_set.discard(addr)
        self._checkNegociating(node)

    def test_connectionClosed(self):
        node, conn = self.identifyToMasterNode()
        self._setNegociating(node)
        self.election.connectionClosed(conn)
        self.assertTrue(node.isUnknown())
        addr = node.getAddress()
        self.assertFalse(addr in self.app.unconnected_master_node_set)
        self.assertFalse(addr in self.app.negotiating_master_node_set)

    def test_acceptIdentification1(self):
        """ A non-master node accept identification """
        node, conn = self.identifyToMasterNode()
        args = (node.getUUID(), 0, 10, self.app.uuid)
        self.election.acceptIdentification(conn,
            NodeTypes.CLIENT, *args)
        self.assertFalse(node in self.app.unconnected_master_node_set)
        self.assertFalse(node in self.app.negotiating_master_node_set)
        self.checkClosed(conn)

    def test_acceptIdentification2(self):
        """ UUID conflict """
        node, conn = self.identifyToMasterNode()
        new_uuid = self._makeUUID('M')
        args = (node.getUUID(), 0, 10, new_uuid)
        self.assertRaises(ElectionFailure, self.election.acceptIdentification,
            conn, NodeTypes.MASTER, *args)
        self.assertEqual(self.app.uuid, new_uuid)

    def test_acceptIdentification3(self):
        """ Identification accepted """
        node, conn = self.identifyToMasterNode()
        args = (node.getUUID(), 0, 10, self.app.uuid)
        self.election.acceptIdentification(conn, NodeTypes.MASTER, *args)
        self.checkUUIDSet(conn, node.getUUID())
        self.assertTrue(self.app.primary or node.getUUID() < self.app.uuid)
        self.assertFalse(node in self.app.negotiating_master_node_set)

    def _getMasterList(self, with_node=None):
        master_list = self.app.nm.getMasterList()
        return [(x.getAddress(), x.getUUID()) for x in master_list]

    def test_answerPrimary1(self):
        """ Multiple primary masters -> election failure raised """
        node, conn = self.identifyToMasterNode()
        self.app.primary = True
        self.app.primary_master_node = node
        master_list = self._getMasterList()
        self.assertRaises(ElectionFailure, self.election.answerPrimary,
                conn, self.app.uuid, master_list)

    def test_answerPrimary2(self):
        """ Don't known who's the primary """
        node, conn = self.identifyToMasterNode()
        master_list = self._getMasterList()
        self.election.answerPrimary(conn, None, master_list)
        self.assertFalse(self.app.primary)
        self.assertEqual(self.app.primary_master_node, None)
        self.checkRequestIdentification(conn)

    def test_answerPrimary3(self):
        """ Answer who's the primary """
        node, conn = self.identifyToMasterNode()
        master_list = self._getMasterList()
        self.election.answerPrimary(conn, node.getUUID(), master_list)
        self.assertEqual(len(self.app.unconnected_master_node_set), 0)
        self.assertEqual(len(self.app.negotiating_master_node_set), 0)
        self.assertFalse(self.app.primary)
        self.assertEqual(self.app.primary_master_node, node)
        self.checkRequestIdentification(conn)


class MasterServerElectionTests(NeoUnitTestBase):

    def setUp(self):
        NeoUnitTestBase.setUp(self)
        # create an application object
        config = self.getMasterConfiguration(master_number=1)
        self.app = Application(config)
        self.app.pt.clear()
        self.app.name = 'NEOCLUSTER'
        self.app.em = Mock()
        self.election = ServerElectionHandler(self.app)
        self.app.unconnected_master_node_set = set()
        self.app.negotiating_master_node_set = set()
        for node in self.app.nm.getMasterList():
            self.app.unconnected_master_node_set.add(node.getAddress())
            node.setState(NodeStates.RUNNING)
        # define some variable to simulate client and storage node
        self.client_address = (self.local_ip, 1000)
        self.storage_address = (self.local_ip, 2000)
        self.master_address = (self.local_ip, 3000)
        # apply monkey patches
        self._addPacket = ClientConnection._addPacket
        ClientConnection._addPacket = _addPacket

    def tearDown(self):
        NeoUnitTestBase.tearDown(self)
        # restore environnement
        ClientConnection._addPacket = self._addPacket

    def identifyToMasterNode(self, uuid=True):
        node = self.app.nm.getMasterList()[0]
        if uuid is True:
            uuid = self.getNewUUID()
        node.setUUID(uuid)
        conn = self.getFakeConnection(
                uuid=node.getUUID(),
                address=node.getAddress(),
        )
        return (node, conn)


    # Tests

    def test_requestIdentification1(self):
        """ A non-master node request identification """
        node, conn = self.identifyToMasterNode()
        args = (node.getUUID(), node.getAddress(), self.app.name)
        self.assertRaises(protocol.NotReadyError,
            self.election.requestIdentification,
            conn, NodeTypes.CLIENT, *args)

    def test_requestIdentification2(self):
        """ A unknown master node request identification """
        node, conn = self.identifyToMasterNode()
        args = (node.getUUID(), ('127.0.0.1', 1000), self.app.name)
        self.checkProtocolErrorRaised(self.election.requestIdentification,
            conn, NodeTypes.MASTER, *args)

    def test_requestIdentification3(self):
        """ A broken master node request identification """
        node, conn = self.identifyToMasterNode()
        node.setBroken()
        args = (node.getUUID(), node.getAddress(), self.app.name)
        self.assertRaises(protocol.BrokenNodeDisallowedError,
            self.election.requestIdentification,
            conn, NodeTypes.MASTER, *args)

    def test_requestIdentification4(self):
        """ No conflict """
        node, conn = self.identifyToMasterNode()
        args = (node.getUUID(), node.getAddress(), self.app.name)
        self.election.requestIdentification(conn,
            NodeTypes.MASTER, *args)
        self.checkUUIDSet(conn, node.getUUID())
        args = self.checkAcceptIdentification(conn, decode=True)
        node_type, uuid, partitions, replicas, new_uuid = args
        self.assertEqual(node.getUUID(), new_uuid)
        self.assertNotEqual(node.getUUID(), uuid)

    def test_requestIdentification5(self):
        """ UUID conflict """
        node, conn = self.identifyToMasterNode()
        args = (self.app.uuid, node.getAddress(), self.app.name)
        self.election.requestIdentification(conn,
            NodeTypes.MASTER, *args)
        self.checkUUIDSet(conn)
        args = self.checkAcceptIdentification(conn, decode=True)
        node_type, uuid, partitions, replicas, new_uuid = args
        self.assertNotEqual(self.app.uuid, new_uuid)
        self.assertEqual(self.app.uuid, uuid)


    def _getNodeList(self):
        return [x.asTuple() for x in self.app.nm.getList()]

    def __getClient(self):
        uuid = self.getNewUUID()
        conn = self.getFakeConnection(uuid=uuid, address=self.client_address)
        self.app.nm.createClient(uuid=uuid, address=self.client_address)
        return conn

    def __getMaster(self, port=1000, register=True):
        uuid = self.getNewUUID()
        address = ('127.0.0.1', port)
        conn = self.getFakeConnection(uuid=uuid, address=address)
        if register:
            self.app.nm.createMaster(uuid=uuid, address=address)
        return conn

    def testRequestIdentification1(self):
        """ Check with a non-master node, must be refused """
        conn = self.__getClient()
        self.checkNotReadyErrorRaised(
            self.election.requestIdentification,
            conn=conn,
            node_type=NodeTypes.CLIENT,
            uuid=conn.getUUID(),
            address=conn.getAddress(),
            name=self.app.name
        )

    def testRequestIdentification2(self):
        """ Check with an unknown master node """
        conn = self.__getMaster(register=False)
        self.checkProtocolErrorRaised(
            self.election.requestIdentification,
            conn=conn,
            node_type=NodeTypes.MASTER,
            uuid=conn.getUUID(),
            address=conn.getAddress(),
            name=self.app.name,
        )

    def testAnnouncePrimary1(self):
        """ check the wrong cases """
        announce = self.election.announcePrimary
        # No uuid
        node, conn = self.identifyToMasterNode(uuid=None)
        self.checkProtocolErrorRaised(announce, conn)
        # Announce to a primary, raise
        self.app.primary = True
        node, conn = self.identifyToMasterNode()
        self.assertTrue(self.app.primary)
        self.assertEqual(self.app.primary_master_node, None)
        self.assertRaises(ElectionFailure, announce, conn)

    def testAnnouncePrimary2(self):
        """ Check the good case """
        announce = self.election.announcePrimary
        # Announce, must set the primary
        self.app.primary = False
        node, conn = self.identifyToMasterNode()
        self.assertFalse(self.app.primary)
        self.assertFalse(self.app.primary_master_node)
        announce(conn)
        self.assertFalse(self.app.primary)
        self.assertEqual(self.app.primary_master_node, node)

    def test_askPrimary1(self):
        """ Ask the primary to the primary """
        node, conn = self.identifyToMasterNode()
        self.app.primary = True
        self.election.askPrimary(conn)
        uuid, master_list = self.checkAnswerPrimary(conn, decode=True)
        self.assertEqual(uuid, self.app.uuid)
        self.assertEqual(len(master_list), 2)
        self.assertEqual(master_list[0], (self.app.server, self.app.uuid))
        master_node = self.app.nm.getMasterList()[0]
        master_node = (master_node.getAddress(), master_node.getUUID())
        self.assertEqual(master_list[1], master_node)

    def test_askPrimary2(self):
        """ Ask the primary to a secondary that known who's te primary """
        node, conn = self.identifyToMasterNode()
        self.app.primary = False
        # it will answer ourself as primary
        self.app.primary_master_node = node
        self.election.askPrimary(conn)
        uuid, master_list = self.checkAnswerPrimary(conn, decode=True)
        self.assertEqual(uuid, node.getUUID())
        self.assertEqual(len(master_list), 2)
        self.assertEqual(master_list[0], (self.app.server, self.app.uuid))
        master_node = (node.getAddress(), node.getUUID())
        self.assertEqual(master_list[1], master_node)

    def test_askPrimary3(self):
        """ Ask the primary to a master that don't known who's the primary """
        node, conn = self.identifyToMasterNode()
        self.app.primary = False
        self.app.primary_master_node = None
        self.election.askPrimary(conn)
        uuid, master_list = self.checkAnswerPrimary(conn, decode=True)
        self.assertEqual(uuid, None)
        self.assertEqual(len(master_list), 2)
        self.assertEqual(master_list[0], (self.app.server, self.app.uuid))
        master_node = self.app.nm.getMasterList()[0]
        master_node = (node.getAddress(), node.getUUID())
        self.assertEqual(master_list[1], master_node)

    def test_reelectPrimary(self):
        node, conn = self.identifyToMasterNode()
        self.assertRaises(ElectionFailure, self.election.reelectPrimary, conn)


if __name__ == '__main__':
    unittest.main()




#
# Copyright (C) 2011  Nexedi SA
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from thread import get_ident
from neo.lib.locking import Queue

class ContainerBase(object):
    def __init__(self):
        self._context_dict = {}

    def _getID(self, *args, **kw):
        raise NotImplementedError

    def _new(self, *args, **kw):
        raise NotImplementedError

    def delete(self, *args, **kw):
        del self._context_dict[self._getID(*args, **kw)]

    def get(self, *args, **kw):
        return self._context_dict.get(self._getID(*args, **kw))

    def new(self, *args, **kw):
        result = self._context_dict[self._getID(*args, **kw)] = self._new(
            *args, **kw)
        return result

class ThreadContainer(ContainerBase):
    def _getID(self):
        return get_ident()

    def _new(self):
        return {
            'queue': Queue(0),
            'answer': None,
        }

    def get(self):
        """
        Implicitely create a thread context if it doesn't exist.
        """
        my_id = self._getID()
        try:
            result = self._context_dict[my_id]
        except KeyError:
            result = self._context_dict[my_id] = self._new()
        return result

class TransactionContainer(ContainerBase):
    def _getID(self, txn):
        return id(txn)

    def _new(self, txn):
        return {
            'queue': Queue(0),
            'txn': txn,
            'ttid': None,
            'data_dict': {},
            'data_list': [],
            'object_base_serial_dict': {},
            'object_serial_dict': {},
            'object_stored_counter_dict': {},
            'conflict_serial_dict': {},
            'resolved_conflict_serial_dict': {},
            'txn_voted': False,
            'involved_nodes': set(),
        }




#
# Copyright (C) 2009-2010  Nexedi SA
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

import unittest
from mock import Mock
from neo.tests import NeoUnitTestBase
from neo.lib.pt import PartitionTable
from neo.lib.protocol import NodeTypes, NodeStates
from neo.client.handlers.master import PrimaryBootstrapHandler
from neo.client.handlers.master import PrimaryNotificationsHandler, \
       PrimaryAnswersHandler
from neo.client.exception import NEOStorageError

MARKER = []


class MasterHandlerTests(NeoUnitTestBase):

    def getConnection(self):
        return self.getFakeConnection()


class MasterBootstrapHandlerTests(MasterHandlerTests):

    def setUp(self):
        NeoUnitTestBase.setUp(self)
        self.app = Mock()
        self.handler = PrimaryBootstrapHandler(self.app)

    def checkCalledOnApp(self, method, index=0):
        calls = self.app.mockGetNamedCalls(method)
        self.assertTrue(len(calls) > index)
        return calls[index].params

    def test_notReady(self):
        conn = self.getConnection()
        self.handler.notReady(conn, 'message')
        self.assertEqual(self.app.trying_master_node, None)

    def test_acceptIdentification1(self):
        """ Non-master node """
        conn = self.getConnection()
        uuid = self.getNewUUID()
        self.handler.acceptIdentification(conn, NodeTypes.CLIENT,
            uuid, 100, 0, None)
        self.checkClosed(conn)

    def test_acceptIdentification2(self):
        """ No UUID supplied """
        conn = self.getConnection()
        uuid = self.getNewUUID()
        self.checkProtocolErrorRaised(self.handler.acceptIdentification,
            conn, NodeTypes.MASTER, uuid, 100, 0, None)

    def test_acceptIdentification3(self):
        """ identification accepted  """
        node = Mock()
        conn = self.getConnection()
        uuid = self.getNewUUID()
        your_uuid = self.getNewUUID()
        partitions = 100
        replicas = 2
        self.app.nm = Mock({'getByAddress': node})
        self.handler.acceptIdentification(conn, NodeTypes.MASTER, uuid,
            partitions, replicas, your_uuid)
        self.assertEqual(self.app.uuid, your_uuid)
        self.checkUUIDSet(conn, uuid)
        self.checkUUIDSet(node, uuid)
        self.assertTrue(isinstance(self.app.pt, PartitionTable))

    def _getMasterList(self, uuid_list):
        port = 1000
        master_list = []
        for uuid in uuid_list:
            master_list.append((('127.0.0.1', port), uuid))
            port += 1
        return master_list

    def test_answerPrimary1(self):
        """ Primary not known, master udpated """
        node, uuid = Mock(), self.getNewUUID()
        conn = self.getConnection()
        master_list = [(('127.0.0.1', 1000), uuid)]
        self.app.primary_master_node = Mock()
        self.app.trying_master_node = Mock()
        self.app.nm = Mock({'getByAddress': node})
        self.handler.answerPrimary(conn, None, master_list)
        self.checkUUIDSet(node, uuid)
        # previously known primary master forgoten
        self.assertEqual(self.app.primary_master_node, None)
        self.assertEqual(self.app.trying_master_node, None)
        self.checkClosed(conn)

    def test_answerPrimary2(self):
        """ Primary known """
        current_node = Mock({'__repr__': '1'})
        node, uuid = Mock({'__repr__': '2'}), self.getNewUUID()
        conn = self.getConnection()
        master_list = [(('127.0.0.1', 1000), uuid)]
        self.app.primary_master_node = None
        self.app.trying_master_node = current_node
        self.app.nm = Mock({
            'getByAddress': node,
            'getByUUID': node,
        })
        self.handler.answerPrimary(conn, uuid, [])
        self.assertEqual(self.app.trying_master_node, None)
        self.assertTrue(self.app.primary_master_node is node)
        self.checkClosed(conn)

    def test_answerPartitionTable(self):
        conn = self.getConnection()
        self.app.pt = Mock()
        ptid = 0
        row_list = ([], [])
        self.handler.answerPartitionTable(conn, ptid, row_list)
        load_calls = self.app.pt.mockGetNamedCalls('load')
        self.assertEqual(len(load_calls), 1)
        # load_calls[0].checkArgs(ptid, row_list, self.app.nm)


class MasterNotificationsHandlerTests(MasterHandlerTests):

    def setUp(self):
        NeoUnitTestBase.setUp(self)
        self.db = Mock()
        self.app = Mock({'getDB': self.db})
        self.app.nm = Mock()
        self.app.dispatcher = Mock()
        self.handler = PrimaryNotificationsHandler(self.app)

    def test_connectionClosed(self):
        conn = self.getConnection()
        node = Mock()
        self.app.master_conn = conn
        self.app.primary_master_node = node
        self.handler.connectionClosed(conn)
        self.assertEqual(self.app.master_conn, None)
        self.assertEqual(self.app.primary_master_node, None)

    def test_invalidateObjects(self):
        conn = self.getConnection()
        tid = self.getNextTID()
        oid1, oid2, oid3 = self.getOID(1), self.getOID(2), self.getOID(3)
        self.app._cache = Mock({
            'invalidate': None,
        })
        self.handler.invalidateObjects(conn, tid, [oid1, oid3])
        cache_calls = self.app._cache.mockGetNamedCalls('invalidate')
        self.assertEqual(len(cache_calls), 2)
        cache_calls[0].checkArgs(oid1, tid)
        cache_calls[1].checkArgs(oid3, tid)
        invalidation_calls = self.db.mockGetNamedCalls('invalidate')
        self.assertEqual(len(invalidation_calls), 1)
        invalidation_calls[0].checkArgs(tid, [oid1, oid3])

    def test_notifyPartitionChanges(self):
        conn = self.getConnection()
        self.app.pt = Mock({'filled': True})
        ptid = 0
        cell_list = (Mock(), Mock())
        self.handler.notifyPartitionChanges(conn, ptid, cell_list)
        update_calls = self.app.pt.mockGetNamedCalls('update')
        self.assertEqual(len(update_calls), 1)
        update_calls[0].checkArgs(ptid, cell_list, self.app.nm)


class MasterAnswersHandlerTests(MasterHandlerTests):

    def setUp(self):
        NeoUnitTestBase.setUp(self)
        self.app = Mock()
        self.handler = PrimaryAnswersHandler(self.app)

    def test_answerBeginTransaction(self):
        tid = self.getNextTID()
        conn = self.getConnection()
        self.handler.answerBeginTransaction(conn, tid)
        calls = self.app.mockGetNamedCalls('setHandlerData')
        self.assertEqual(len(calls), 1)
        calls[0].checkArgs(tid)

    def test_answerNewOIDs(self):
        conn = self.getConnection()
        oid1, oid2, oid3 = self.getOID(0), self.getOID(1), self.getOID(2)
        self.handler.answerNewOIDs(conn, [oid1, oid2, oid3])
        self.assertEqual(self.app.new_oid_list, [oid1, oid2, oid3])

    def test_answerTransactionFinished(self):
        conn = self.getConnection()
        ttid2 = self.getNextTID()
        tid2 = self.getNextTID()
        self.handler.answerTransactionFinished(conn, ttid2, tid2)
        calls = self.app.mockGetNamedCalls('setHandlerData')
        self.assertEqual(len(calls), 1)
        calls[0].checkArgs(tid2)

    def test_answerPack(self):
        self.assertRaises(NEOStorageError, self.handler.answerPack, None, False)
        # Check it doesn't raise
        self.handler.answerPack(None, True)

if __name__ == '__main__':
    unittest.main()




#
# Copyright (c) 2011 Nexedi SARL and Contributors. All Rights Reserved.
#                    Julien Muchembled <jm@nexedi.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

import __builtin__
import errno
import mmap
import os
import psutil
import signal
import socket
import sys
import tempfile
from cPickle import dumps, loads
from functools import wraps
from time import time, sleep
from neo.lib import debug


class SocketLock(object):
    """Basic system-wide lock"""

    _socket = None

    def __init__(self, address, family=socket.AF_UNIX, type=socket.SOCK_DGRAM):
        if family == socket.AF_UNIX:
            address = '\0' + address
        self.address = address
        self.socket_args = family, type

    def locked(self):
        return self._socket is not None

    def acquire(self, blocking=1):
        assert self._socket is None
        s = socket.socket(*self.socket_args)
        try:
            while True:
                try:
                    s.bind(self.address)
                except socket.error, e:
                    if e[0] != errno.EADDRINUSE:
                        raise
                    if not blocking:
                        return False
                    sleep(1)
                else:
                    self._socket = s
                    return True
        finally:
            if self._socket is None:
                s.close()

    def release(self):
        s = self._socket
        del self._socket
        s.close()


class ClusterDict(dict):
    """Simple storage (dict), shared with forked processes"""

    _acquired = 0

    def __init__(self, *args, **kw):
        dict.__init__(self, *args, **kw)
        self._r, self._w = os.pipe()
        # shm_open(3) would be better but Python doesn't provide it.
        # See also http://nikitathespider.com/python/shm/
        f = tempfile.TemporaryFile()
        try:
            f.write(dumps(self.copy(), -1))
            f.flush()
            self._shared = mmap.mmap(f.fileno(), f.tell())
        finally:
            f.close()
        self.release()

    def __del__(self):
        try:
            os.close(self._r)
            os.close(self._w)
        except TypeError: # if os.close is None
            pass

    def acquire(self):
        self._acquired += 1
        if not self._acquired:
            os.read(self._r, 1)
            try:
                self.clear()
                shared = self._shared
                shared.resize(shared.size())
                self.update(loads(shared[:]))
            except:
                self.release()
                raise

    def release(self, commit=False):
        if not self._acquired:
            if commit:
                self.commit()
            os.write(self._w, '\0')
        self._acquired -= 1

    def commit(self):
        shared = self._shared
        p = dumps(self.copy(), -1)
        shared.resize(len(p))
        shared[:] = p

cluster_dict = ClusterDict()


class ClusterPdb(object):
    """Multiprocess-aware wrapper around console and winpdb debuggers

    __call__ is the method to break.

    TODO: monkey-patch normal code not to timeout
          if another node is being debugged
    """

    def __init__(self):
        self._count_dict = {}

    def __setattr__(self, name, value):
        try:
            hook = getattr(self, name)
            setattr(value.im_self, value.__name__, wraps(value)(
                lambda *args, **kw: hook(value, *args, **kw)))
        except AttributeError:
            object.__setattr__(self, name, value)

    @property
    def broken_peer(self):
        return self._getLastPdb(os.getpid()) is None

    def __call__(self, max_count=None, depth=0, text=None):
        depth += 1
        if max_count:
            frame = sys._getframe(depth)
            key = id(frame.f_code), frame.f_lineno
            del frame
            self._count_dict[key] = count = 1 + self._count_dict.get(key, 0)
            if max_count < count:
                return
        if not text:
            try:
                import rpdb2
            except ImportError:
                if text is not None:
                    raise
            else:
                if rpdb2.g_debugger is None:
                    rpdb2_CStateManager = rpdb2.CStateManager
                    def CStateManager(*args, **kw):
                        rpdb2.CStateManager = rpdb2_CStateManager
                        state_manager = rpdb2.CStateManager(*args, **kw)
                        self._rpdb2_set_state = state_manager.set_state
                        return state_manager
                    rpdb2.CStateManager = CStateManager
                return debug.winpdb(depth)
        try:
            debugger = self.__dict__['_debugger']
        except KeyError:
            assert 'rpdb2' not in sys.modules
            self._debugger = debugger = debug.getPdb()
            self._bdb_interaction = debugger.interaction
        return debugger.set_trace(sys._getframe(depth))

    def kill(self, pid, sig):
        force = []
        sigint_handler = None
        try:
            while 1:
                cluster_dict.acquire()
                try:
                    last_pdb = cluster_dict.get('last_pdb', {})
                    if force or pid not in last_pdb:
                        os.kill(pid, sig)
                        last_pdb.pop(pid, None)
                        cluster_dict.commit()
                        break
                    try:
                        if psutil.Process(pid).status == psutil.STATUS_ZOMBIE:
                            break
                    except psutil.NoSuchProcess:
                        raise OSError(errno.ESRCH, 'No such process')
                finally:
                    cluster_dict.release()
                if sigint_handler is None:
                    sigint_handler = signal.signal(signal.SIGINT,
                        lambda *args: force.append(None))
                    sys.stderr.write('Pid %u is/was debugged.'
                                    ' Press ^C to kill it...' % pid)
                sleep(1)
        finally:
            if sigint_handler is not None:
                signal.signal(signal.SIGINT, sigint_handler)
        if force:
            sys.stderr.write('\n')

    def _lock_console(self):
        while 1:
            cluster_dict.acquire()
            try:
                if 'text_pdb' not in cluster_dict:
                    cluster_dict['text_pdb'] = pid = os.getpid()
                    cluster_dict.setdefault('last_pdb', {})[pid] = None
                    cluster_dict.commit()
                    break
            finally:
                cluster_dict.release()
            sleep(0.5)

    def _unlock_console(self):
        cluster_dict.acquire()
        try:
            pid = cluster_dict.pop('text_pdb')
            cluster_dict['last_pdb'][pid] = time()
            cluster_dict.commit()
        finally:
            cluster_dict.release()

    def _bdb_interaction(self, hooked, *args, **kw):
        self._lock_console()
        try:
            return hooked(*args, **kw)
        finally:
            self._unlock_console()

    def _rpdb2_set_state(self, hooked, state=None, *args, **kw):
        from rpdb2 import STATE_BROKEN, STATE_DETACHED
        cluster_dict.acquire()
        try:
            if state is None:
                state = hooked.im_self.get_state()
            last_pdb = cluster_dict.setdefault('last_pdb', {})
            pid = os.getpid()
            if state == STATE_DETACHED:
                last_pdb.pop(pid, None)
            else:
                last_pdb[pid] = state != STATE_BROKEN and time() or None
            return hooked(state=state, *args, **kw)
        finally:
            cluster_dict.release(True)

    def _getLastPdb(self, *exclude):
        result = 0
        for pid, last_pdb in cluster_dict.get('last_pdb', {}).iteritems():
            if pid not in exclude:
                if last_pdb is None:
                    return
                if result < last_pdb:
                    result = last_pdb
        return result

    def wait(self, test, timeout):
        end_time = time() + timeout
        period = 0.1
        while not test():
            cluster_dict.acquire()
            try:
                last_pdb = self._getLastPdb()
                if last_pdb is None:
                    next_sleep = 1
                else:
                    next_sleep = max(last_pdb + timeout, end_time) - time()
                    if next_sleep > period:
                        next_sleep = period
                        period *= 1.5
                    elif next_sleep < 0:
                        return False
            finally:
                cluster_dict.release()
            sleep(next_sleep)
        return True

__builtin__.pdb = ClusterPdb()

signal.signal(signal.SIGUSR2, debug.decorate(lambda sig, frame: pdb(depth=2)))



#
# Copyright (C) 2006-2010  Nexedi SA
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from time import time
import neo.lib
from neo.lib.epoll import Epoll
from neo.lib.profiling import profiler_decorator

class EpollEventManager(object):
    """This class manages connections and events based on epoll(5)."""

    def __init__(self):
        self.connection_dict = {}
        self.reader_set = set([])
        self.writer_set = set([])
        self.epoll = Epoll()
        self._pending_processing = []

    def close(self):
        for c in self.connection_dict.values():
            c.close()
        del self.__dict__

    def getConnectionList(self):
        # XXX: use index
        return [x for x in self.connection_dict.values() if not x.isAborted()]

    def getClientList(self):
        # XXX: use index
        return [c for c in self.getConnectionList() if c.isClient()]

    def getServerList(self):
        # XXX: use index
        return [c for c in self.getConnectionList() if c.isServer()]

    def getConnectionListByUUID(self, uuid):
        """ Return the connection associated to the UUID, None if the UUID is
        None, invalid or not found"""
        # XXX: use index
        # XXX: consider remove UUID from connection and thus this method
        if uuid is None:
            return None
        result = []
        append = result.append
        for conn in self.getConnectionList():
            if conn.getUUID() == uuid:
                append(conn)
        return result

    def register(self, conn):
        fd = conn.getConnector().getDescriptor()
        self.connection_dict[fd] = conn
        self.epoll.register(fd)

    def unregister(self, conn):
        new_pending_processing = [x for x in self._pending_processing
                                  if x is not conn]
        # Check that we removed at most one entry from
        # self._pending_processing .
        assert len(new_pending_processing) > len(self._pending_processing) - 2
        self._pending_processing = new_pending_processing
        fd = conn.getConnector().getDescriptor()
        self.epoll.unregister(fd)
        del self.connection_dict[fd]

    def _getPendingConnection(self):
        if len(self._pending_processing):
            result = self._pending_processing.pop(0)
        else:
            result = None
        return result

    def _addPendingConnection(self, conn):
        pending_processing = self._pending_processing
        if conn not in pending_processing:
            pending_processing.append(conn)

    def poll(self, timeout=1):
        to_process = self._getPendingConnection()
        if to_process is None:
            # Fetch messages from polled file descriptors
            self._poll(timeout=timeout)
            # See if there is anything to process
            to_process = self._getPendingConnection()
        if to_process is not None:
            to_process.lock()
            try:
                try:
                    # Process
                    to_process.process()
                finally:
                    # ...and requeue if there are pending messages
                    if to_process.hasPendingMessages():
                        self._addPendingConnection(to_process)
            finally:
                to_process.unlock()
            # Non-blocking call: as we handled a packet, we should just offer
            # poll a chance to fetch & send already-available data, but it must
            # not delay us.
            self._poll(timeout=0)

    def _poll(self, timeout=1):
        rlist, wlist, elist = self.epoll.poll(timeout)
        for fd in frozenset(rlist):
            conn = self.connection_dict[fd]
            conn.lock()
            try:
                conn.readable()
            finally:
                conn.unlock()
            if conn.hasPendingMessages():
                self._addPendingConnection(conn)

        for fd in frozenset(wlist):
            # This can fail, if a connection is closed in readable().
            try:
                conn = self.connection_dict[fd]
            except KeyError:
                pass
            else:
                conn.lock()
                try:
                    conn.writable()
                finally:
                    conn.unlock()

        for fd in frozenset(elist):
            # This can fail, if a connection is closed in previous calls to
            # readable() or writable().
            try:
                conn = self.connection_dict[fd]
            except KeyError:
                pass
            else:
                conn.lock()
                try:
                    conn.readable()
                finally:
                    conn.unlock()
            if conn.hasPendingMessages():
                self._addPendingConnection(conn)

        t = time()
        for conn in self.connection_dict.values():
            conn.lock()
            try:
                conn.checkTimeout(t)
            finally:
                conn.unlock()

    def addReader(self, conn):
        connector = conn.getConnector()
        assert connector is not None, conn.whoSetConnector()
        fd = connector.getDescriptor()
        if fd not in self.reader_set:
            self.reader_set.add(fd)
            self.epoll.modify(fd, 1, fd in self.writer_set)

    def removeReader(self, conn):
        connector = conn.getConnector()
        assert connector is not None, conn.whoSetConnector()
        fd = connector.getDescriptor()
        if fd in self.reader_set:
            self.reader_set.remove(fd)
            self.epoll.modify(fd, 0, fd in self.writer_set)

    @profiler_decorator
    def addWriter(self, conn):
        connector = conn.getConnector()
        assert connector is not None, conn.whoSetConnector()
        fd = connector.getDescriptor()
        if fd not in self.writer_set:
            self.writer_set.add(fd)
            self.epoll.modify(fd, fd in self.reader_set, 1)

    def removeWriter(self, conn):
        connector = conn.getConnector()
        assert connector is not None, conn.whoSetConnector()
        fd = connector.getDescriptor()
        if fd in self.writer_set:
            self.writer_set.remove(fd)
            self.epoll.modify(fd, fd in self.reader_set, 0)

    def log(self):
        neo.lib.logging.info('Event Manager:')
        neo.lib.logging.info('  Readers: %r', [x for x in self.reader_set])
        neo.lib.logging.info('  Writers: %r', [x for x in self.writer_set])
        neo.lib.logging.info('  Connections:')
        pending_set = set(self._pending_processing)
        for fd, conn in self.connection_dict.items():
            neo.lib.logging.info('    %r: %r (pending=%r)', fd, conn,
                conn in pending_set)


# Default to EpollEventManager.
EventManager = EpollEventManager



# Copyright (C) 2011  Nexedi SA
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

import sys, types

if sys.version_info < (2, 5):
    import __builtin__, imp

    def all(iterable):
      """
      Return True if bool(x) is True for all values x in the iterable.
      """
      for x in iterable:
        if not x:
          return False
      return True
    __builtin__.all = all

    def any(iterable):
      """
      Return True if bool(x) is True for any x in the iterable.
      """
      for x in iterable:
        if x:
          return True
      return False
    __builtin__.any = any

    import md5, sha
    sys.modules['hashlib'] = hashlib = imp.new_module('hashlib')
    hashlib.md5 = md5.new
    hashlib.sha1 = sha.new

    import struct

    class Struct(object):

        def __init__(self, fmt):
            self._fmt = fmt
            self.size = struct.calcsize(fmt)

        def pack(self, *args):
            return struct.pack(self._fmt, *args)

        def unpack(self, *args):
            return struct.unpack(self._fmt, *args)

    struct.Struct = Struct

    sys.modules['functools'] = functools = imp.new_module('functools')

    def wraps(wrapped):
        """Simple backport of functools.wraps from Python >= 2.5"""
        def decorator(wrapper):
            wrapper.__module__ = wrapped.__module__
            wrapper.__name__ = wrapped.__name__
            wrapper.__doc__ = wrapped.__doc__
            wrapper.__dict__.update(wrapped.__dict__)
            return wrapper
        return decorator

    functools.wraps = wraps



#
# Copyright (C) 2006-2010  Nexedi SA
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

import neo.lib

from neo.storage.handlers import BaseMasterHandler
from neo.lib.protocol import CellStates

class HiddenHandler(BaseMasterHandler):
    """This class implements a generic part of the event handlers."""

    def notifyPartitionChanges(self, conn, ptid, cell_list):
        """This is very similar to Send Partition Table, except that
        the information is only about changes from the previous."""
        app = self.app
        if ptid <= app.pt.getID():
            # Ignore this packet.
            neo.lib.logging.debug('ignoring older partition changes')
            return

        # update partition table in memory and the database
        app.pt.update(ptid, cell_list, app.nm)
        app.dm.changePartitionTable(ptid, cell_list)

        # Check changes for replications
        for offset, uuid, state in cell_list:
            if uuid == app.uuid and app.replicator is not None:
                # If this is for myself, this can affect replications.
                if state == CellStates.DISCARDED:
                    app.replicator.removePartition(offset)
                elif state == CellStates.OUT_OF_DATE:
                    app.replicator.addPartition(offset)

    def startOperation(self, conn):
        self.app.operational = True




#
# Copyright (C) 2009-2010  Nexedi SA
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

import unittest
from mock import Mock
from neo.tests import NeoUnitTestBase
from neo.storage.app import Application
from neo.lib.bootstrap import BootstrapManager
from neo.lib.protocol import NodeTypes

class BootstrapManagerTests(NeoUnitTestBase):

    def setUp(self):
        NeoUnitTestBase.setUp(self)
        self.prepareDatabase(number=1)
        # create an application object
        config = self.getStorageConfiguration()
        self.app = Application(config)
        self.bootstrap = BootstrapManager(self.app, 'main', NodeTypes.STORAGE)
        # define some variable to simulate client and storage node
        self.master_port = 10010
        self.storage_port = 10020
        self.num_partitions = 1009
        self.num_replicas = 2

    def tearDown(self):
        self.app.close()
        del self.app
        super(BootstrapManagerTests, self).tearDown()

    # Common methods
    def getLastUUID(self):
        return self.uuid

    # Tests
    def testConnectionCompleted(self):
        address=("127.0.0.1", self.master_port)
        conn = self.getFakeConnection(address=address)
        self.bootstrap.current = self.app.nm.createMaster(address=address)
        self.bootstrap.connectionCompleted(conn)
        self.checkAskPrimary(conn)

    def testHandleNotReady(self):
        # the primary is not ready
        address=("127.0.0.1", self.master_port)
        conn = self.getFakeConnection(address=address)
        self.bootstrap.current = self.app.nm.createMaster(address=address)
        self.bootstrap.notReady(conn, '')
        self.checkClosed(conn)
        self.checkNoPacketSent(conn)


if __name__ == "__main__":
    unittest.main()




#
# Copyright (C) 2009-2010  Nexedi SA
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

import unittest
import transaction
from persistent import Persistent

from neo.tests.functional import NEOCluster, NEOFunctionalTest

class ClusterTests(NEOFunctionalTest):

    def setUp(self):
        NEOFunctionalTest.setUp(self)
        self.neo = None

    def tearDown(self):
        if self.neo is not None:
            self.neo.stop()
        NEOFunctionalTest.tearDown(self)

    def testClusterBreaks(self):
        self.neo = NEOCluster(['test_neo1'],
                master_count=1, temp_dir=self.getTempDirectory())
        neoctl = self.neo.getNEOCTL()
        self.neo.setupDB()
        self.neo.start()
        self.neo.expectClusterRunning()
        self.neo.expectOudatedCells(number=0)
        self.neo.killStorage()
        self.neo.expectClusterVerifying()

    def testClusterBreaksWithTwoNodes(self):
        self.neo = NEOCluster(['test_neo1', 'test_neo2'],
                 partitions=2, master_count=1, replicas=0,
                 temp_dir=self.getTempDirectory())
        neoctl = self.neo.getNEOCTL()
        self.neo.setupDB()
        self.neo.start()
        self.neo.expectClusterRunning()
        self.neo.expectOudatedCells(number=0)
        self.neo.killStorage()
        self.neo.expectClusterVerifying()

    def testClusterDoesntBreakWithTwoNodesOneReplica(self):
        self.neo = NEOCluster(['test_neo1', 'test_neo2'],
                         partitions=2, replicas=1, master_count=1,
                         temp_dir=self.getTempDirectory())
        neoctl = self.neo.getNEOCTL()
        self.neo.setupDB()
        self.neo.start()
        self.neo.expectClusterRunning()
        self.neo.expectOudatedCells(number=0)
        self.neo.killStorage()
        self.neo.expectClusterRunning()

    def testElectionWithManyMasters(self):
        MASTER_COUNT = 20
        self.neo = NEOCluster(['test_neo1', 'test_neo2'],
            partitions=10, replicas=0, master_count=MASTER_COUNT,
            temp_dir=self.getTempDirectory())
        neoctl = self.neo.getNEOCTL()
        self.neo.start()
        self.neo.expectClusterRunning()
        self.neo.expectAllMasters(MASTER_COUNT)
        self.neo.expectOudatedCells(0)

    def testLeavingOperationalStateDropClientNodes(self):
        """
            Check that client nodes are dropped where the cluster leaves the
            operational state.
        """
        # start a cluster
        self.neo = NEOCluster(['test_neo1'], replicas=0,
            temp_dir=self.getTempDirectory())
        neoctl = self.neo.getNEOCTL()
        self.neo.start()
        self.neo.expectClusterRunning()
        self.neo.expectOudatedCells(0)
        # connect a client a check it's known
        db, conn = self.neo.getZODBConnection()
        self.assertEqual(len(self.neo.getClientlist()), 1)
        # drop the storage, the cluster is no more operational...
        self.neo.getStorageProcessList()[0].stop()
        self.neo.expectClusterVerifying()
        # ...and the client gets disconnected
        self.assertEqual(len(self.neo.getClientlist()), 0)
        # restart storage so that the cluster is operational again
        self.neo.getStorageProcessList()[0].start()
        self.neo.expectClusterRunning()
        self.neo.expectOudatedCells(0)
        # and reconnect the client, there must be only one known by the admin
        conn.root()['plop'] = 1
        transaction.commit()
        self.assertEqual(len(self.neo.getClientlist()), 1)

    def testStorageLostDuringRecovery(self):
        """
            Check that admin node receive notifications of storage
            connection and disconnection during recovery
        """
        self.neo = NEOCluster(['test_neo%d' % i for i in xrange(2)],
            master_count=1, partitions=10, replicas=1,
            temp_dir=self.getTempDirectory(), clear_databases=True,
        )
        storages  = self.neo.getStorageProcessList()
        self.neo.run(except_storages=storages)
        self.neo.expectStorageNotKnown(storages[0])
        self.neo.expectStorageNotKnown(storages[1])
        storages[0].start()
        self.neo.expectRunning(storages[0])
        self.neo.expectStorageNotKnown(storages[1])
        storages[1].start()
        self.neo.expectRunning(storages[0])
        self.neo.expectRunning(storages[1])
        storages[0].stop()
        self.neo.expectUnavailable(storages[0])
        self.neo.expectRunning(storages[1])
        storages[1].stop()
        self.neo.expectUnavailable(storages[0])
        self.neo.expectUnavailable(storages[1])

def test_suite():
    return unittest.makeSuite(ClusterTests)

if __name__ == "__main__":
    unittest.main(defaultTest="test_suite")




#
# Copyright (C) 2006-2010  Nexedi SA
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

import neo.lib

from neo.lib.protocol import NodeTypes, NodeStates, Packets
from neo.lib.protocol import NotReadyError, ProtocolError, \
                              UnexpectedPacketError
from neo.lib.protocol import BrokenNodeDisallowedError
from neo.master.handlers import MasterHandler
from neo.lib.exception import ElectionFailure
from neo.lib.util import dump

class ClientElectionHandler(MasterHandler):

    # FIXME: this packet is not allowed here, but handled in MasterHandler
    # a global handler review is required.
    def askPrimary(self, conn):
        raise UnexpectedPacketError, "askPrimary on server connection"

    def connectionStarted(self, conn):
        addr = conn.getAddress()
        # connection in progress
        self.app.unconnected_master_node_set.remove(addr)
        self.app.negotiating_master_node_set.add(addr)
        MasterHandler.connectionStarted(self, conn)

    def connectionFailed(self, conn):
        addr = conn.getAddress()
        node = self.app.nm.getByAddress(addr)
        assert node is not None, (dump(self.app.uuid), addr)
        assert node.isUnknown(), (dump(self.app.uuid), node.whoSetState(),
          node.getState())
        # connection never success, node is still in unknown state
        self.app.negotiating_master_node_set.discard(addr)
        self.app.unconnected_master_node_set.add(addr)
        MasterHandler.connectionFailed(self, conn)

    def connectionCompleted(self, conn):
        conn.ask(Packets.AskPrimary())
        MasterHandler.connectionCompleted(self, conn)

    def connectionLost(self, conn, new_state):
        addr = conn.getAddress()
        self.app.negotiating_master_node_set.discard(addr)

    def acceptIdentification(self, conn, node_type,
            uuid, num_partitions, num_replicas, your_uuid):
        app = self.app
        node = app.nm.getByAddress(conn.getAddress())
        if node_type != NodeTypes.MASTER:
            # The peer is not a master node!
            neo.lib.logging.error('%r is not a master node', conn)
            app.nm.remove(node)
            conn.close()
            return

        if your_uuid != app.uuid:
            # uuid conflict happened, accept the new one and restart election
            app.uuid = your_uuid
            neo.lib.logging.info('UUID conflict, new UUID: %s',
                            dump(your_uuid))
            raise ElectionFailure, 'new uuid supplied'

        conn.setUUID(uuid)
        node.setUUID(uuid)

        if app.uuid < uuid:
            # I lost.
            app.primary = False

        app.negotiating_master_node_set.discard(conn.getAddress())

    def answerPrimary(self, conn, primary_uuid, known_master_list):
        app = self.app
        # Register new master nodes.
        for address, uuid in known_master_list:
            if app.server == address:
                # This is self.
                continue
            n = app.nm.getByAddress(address)
            # master node must be known
            assert n is not None, 'Unknown master node: %s' % (address, )
            if uuid is not None:
                # If I don't know the UUID yet, believe what the peer
                # told me at the moment.
                if n.getUUID() is None or n.getUUID() != uuid:
                    n.setUUID(uuid)

        if primary_uuid is not None:
            # The primary master is defined.
            if app.primary_master_node is not None \
                    and app.primary_master_node.getUUID() != primary_uuid:
                # There are multiple primary master nodes. This is
                # dangerous.
                raise ElectionFailure, 'multiple primary master nodes'
            primary_node = app.nm.getByUUID(primary_uuid)
            if primary_node is None:
                # I don't know such a node. Probably this information
                # is old. So ignore it.
                neo.lib.logging.warning(
                                'received an unknown primary node UUID')
            else:
                # Whatever the situation is, I trust this master.
                app.primary = False
                app.primary_master_node = primary_node
                # Stop waiting for connections than primary master's to
                # complete to exit election phase ASAP.
                app.unconnected_master_node_set.clear()
                app.negotiating_master_node_set.clear()

        primary_node = app.primary_master_node
        if (primary_node is None or \
            conn.getAddress() == primary_node.getAddress()) and \
                not conn.isClosed():
            # Request a node identification.
            # There are 3 cases here:
            # - Peer doesn't know primary node
            #   We must ask its identification so we exchange our uuids, to
            #   know which of us is secondary.
            # - Peer knows primary node
            #   - He is the primary
            #     We must ask its identification, as part of the normal
            #     connection process
            #   - He is not the primary
            #     We don't need to ask its identification, as we will close
            #     this connection anyway (exiting election).
            # Also, connection can be closed by peer after he sent
            # AnswerPrimary if he finds the primary master before we
            # give him our UUID.
            # The connection gets closed before this message gets processed
            # because this message might have been queued, but connection
            # interruption takes effect as soon as received.
            conn.ask(Packets.RequestIdentification(
                NodeTypes.MASTER,
                app.uuid,
                app.server,
                app.name
            ))


class ServerElectionHandler(MasterHandler):

    def reelectPrimary(self, conn):
        raise ElectionFailure, 'reelection requested'

    def requestIdentification(self, conn, node_type,
                                        uuid, address, name):
        self.checkClusterName(name)
        app = self.app
        if node_type != NodeTypes.MASTER:
            neo.lib.logging.info('reject a connection from a non-master')
            raise NotReadyError
        node = app.nm.getByAddress(address)
        if node is None:
            neo.lib.logging.error('unknown master node: %s' % (address, ))
            raise ProtocolError('unknown master node')
        # If this node is broken, reject it.
        if node.getUUID() == uuid:
            if node.isBroken():
                raise BrokenNodeDisallowedError

        # supplied another uuid in case of conflict
        while not app.isValidUUID(uuid, address):
            uuid = app.getNewUUID(node_type)

        node.setUUID(uuid)
        conn.setUUID(uuid)

        p = Packets.AcceptIdentification(
            NodeTypes.MASTER,
            app.uuid,
            app.pt.getPartitions(),
            app.pt.getReplicas(),
            uuid
        )
        conn.answer(p)

    def announcePrimary(self, conn):
        uuid = conn.getUUID()
        if uuid is None:
            raise ProtocolError('Not identified')
        app = self.app
        if app.primary:
            # I am also the primary... So restart the election.
            raise ElectionFailure, 'another primary arises'
        node = app.nm.getByUUID(uuid)
        app.primary = False
        app.primary_master_node = node
        app.unconnected_master_node_set.clear()
        app.negotiating_master_node_set.clear()
        neo.lib.logging.info('%s is the primary', node)




#
# Copyright (C) 2009-2010  Nexedi SA
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

import unittest
from mock import Mock, ReturnValues

from neo.tests import NeoUnitTestBase
from neo.client.app import ConnectionPool
from neo.client.exception import NEOStorageError

class ConnectionPoolTests(NeoUnitTestBase):

    def test_removeConnection(self):
        app = None
        pool = ConnectionPool(app)
        test_node_uuid = self.getNewUUID()
        other_node_uuid = test_node_uuid
        while other_node_uuid == test_node_uuid:
            other_node_uuid = self.getNewUUID()
        test_node = Mock({'getUUID': test_node_uuid})
        other_node = Mock({'getUUID': other_node_uuid})
        # Test sanity check
        self.assertEqual(getattr(pool, 'connection_dict', None), {})
        # Call must not raise if node is not known
        self.assertEqual(len(pool.connection_dict), 0)
        pool.removeConnection(test_node)
        # Test that removal with another uuid doesn't affect entry
        pool.connection_dict[test_node_uuid] = None
        self.assertEqual(len(pool.connection_dict), 1)
        pool.removeConnection(other_node)
        self.assertEqual(len(pool.connection_dict), 1)
        # Test that removeConnection works
        pool.removeConnection(test_node)
        self.assertEqual(len(pool.connection_dict), 0)

    # TODO: test getConnForNode (requires splitting complex functionalities)

    def test_CellSortKey(self):
        pool = ConnectionPool(None)
        node_uuid_1 = self.getNewUUID()
        node_uuid_2 = self.getNewUUID()
        node_uuid_3 = self.getNewUUID()
        # We are connected to node 1
        pool.connection_dict[node_uuid_1] = None
        # A connection to node 3 failed, will be forgotten at 5
        pool._notifyFailure(node_uuid_3, 5)
        getCellSortKey = pool._getCellSortKey

        # At 0, key values are not ambiguous
        self.assertTrue(getCellSortKey(node_uuid_1, 0) < getCellSortKey(
            node_uuid_2, 0) < getCellSortKey(node_uuid_3, 0))
        # At 10, nodes 2 and 3 have the same key value
        self.assertTrue(getCellSortKey(node_uuid_1, 10) < getCellSortKey(
            node_uuid_2, 10))
        self.assertEqual(getCellSortKey(node_uuid_2, 10), getCellSortKey(
            node_uuid_3, 10))

    def test_iterateForObject_noStorageAvailable(self):
        # no node available
        oid = self.getOID(1)
        pt = Mock({'getCellListForOID': []})
        app = Mock({'getPartitionTable': pt})
        pool = ConnectionPool(app)
        self.assertRaises(NEOStorageError, pool.iterateForObject(oid).next)

    def test_iterateForObject_connectionRefused(self):
        # connection refused at the first try
        oid = self.getOID(1)
        node = Mock({'__repr__': 'node', 'isRunning': True})
        cell = Mock({'__repr__': 'cell', 'getNode': node})
        conn = Mock({'__repr__': 'conn'})
        pt = Mock({'getCellListForOID': [cell]})
        app = Mock({'getPartitionTable': pt})
        pool = ConnectionPool(app)
        pool.getConnForNode = Mock({'__call__': ReturnValues(None, conn)})
        self.assertEqual(list(pool.iterateForObject(oid)), [(node, conn)])

    def test_iterateForObject_connectionAccepted(self):
        # connection accepted
        oid = self.getOID(1)
        node = Mock({'__repr__': 'node', 'isRunning': True})
        cell = Mock({'__repr__': 'cell', 'getNode': node})
        conn = Mock({'__repr__': 'conn'})
        pt = Mock({'getCellListForOID': [cell]})
        app = Mock({'getPartitionTable': pt})
        pool = ConnectionPool(app)
        pool.getConnForNode = Mock({'__call__': conn})
        self.assertEqual(list(pool.iterateForObject(oid)), [(node, conn)])

if __name__ == '__main__':
    unittest.main()




#
# Copyright (C) 2006-2010  Nexedi SA
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from ZODB import BaseStorage, ConflictResolution, POSException
from zope.interface import implements
import ZODB.interfaces

import neo.lib
from functools import wraps
from neo.lib import setupLog
from neo.lib.util import add64
from neo.lib.protocol import ZERO_TID
from neo.client.app import Application
from neo.client.exception import NEOStorageNotFoundError
from neo.client.exception import NEOStorageDoesNotExistError

def check_read_only(func):
    def wrapped(self, *args, **kw):
        if self._is_read_only:
            raise POSException.ReadOnlyError()
        return func(self, *args, **kw)
    return wraps(func)(wrapped)

class Storage(BaseStorage.BaseStorage,
              ConflictResolution.ConflictResolvingStorage):
    """Wrapper class for neoclient."""

    # Stores the highest TID visible for current transaction.
    # First call sets this snapshot by asking master node most recent
    # committed TID.
    # As a (positive) side-effect, this forces us to handle all pending
    # invalidations, so we get a very recent view of the database (which is
    # good when multiple databases are used in the same program with some
    # amount of referential integrity).
    # Should remain None when not bound to a connection,
    # so that it always read the last revision.
    _snapshot_tid = None

    implements(*filter(None, (
        ZODB.interfaces.IStorage,
        # "restore" missing for the moment, but "store" implements this
        # interface.
        # ZODB.interfaces.IStorageRestoreable,
        # XXX: imperfect iterator implementation:
        # - start & stop are not handled (raises if either is not None)
        # - transaction isolation is not done
        # ZODB.interfaces.IStorageIteration,
        ZODB.interfaces.IStorageUndoable,
        getattr(ZODB.interfaces, 'IExternalGC', None), # XXX ZODB < 3.9
        getattr(ZODB.interfaces, 'ReadVerifyingStorage', None), # XXX ZODB 3.9
    )))

    def __init__(self, master_nodes, name, read_only=False,
            compress=None, logfile=None, verbose=False, _app=None, **kw):
        """
        Do not pass those parameters (used internally):
        _app
        _cache
        """
        if compress is None:
            compress = True
        setupLog('CLIENT', filename=logfile, verbose=verbose)
        BaseStorage.BaseStorage.__init__(self, 'NEOStorage(%s)' % (name, ))
        # Warning: _is_read_only is used in BaseStorage, do not rename it.
        self._is_read_only = read_only
        if _app is None:
            _app = Application(master_nodes, name, compress=compress)
        self.app = _app
        # Used to clone self (see new_instance & IMVCCStorage definition).
        self._init_args = (master_nodes, name)
        self._init_kw = {
            'read_only': read_only,
            'compress': compress,
            'logfile': logfile,
            'verbose': verbose,
            '_app': _app,
        }

    @property
    def _cache(self):
        return self.app._cache

    def load(self, oid, version=''):
        # In order to know if it was safe to get the last revision of an object
        # instead of using loadBefore(), ZODB.Connection._setstate relies on
        # the fact that retrieving data from a remote storage forces incoming
        # invalidations to be received.
        # But in NEO, invalidations are not received from the same network
        # connection that the one used to retrieve data.
        # So we must implement load() like a loadBefore().
        # XXX: interface definition states that version parameter is
        # mandatory, while some ZODB tests do not provide it. For now, make
        # it optional.
        assert version == '', 'Versions are not supported'
        try:
            return self.app.load(oid, None, self._snapshot_tid)[:2]
        except NEOStorageNotFoundError:
            raise POSException.POSKeyError(oid)

    @check_read_only
    def new_oid(self):
        return self.app.new_oid()

    @check_read_only
    def tpc_begin(self, transaction, tid=None, status=' '):
        """
        Note: never blocks in NEO.
        """
        return self.app.tpc_begin(transaction=transaction, tid=tid,
                status=status)

    @check_read_only
    def tpc_vote(self, transaction):
        return self.app.tpc_vote(transaction=transaction,
            tryToResolveConflict=self.tryToResolveConflict)

    @check_read_only
    def tpc_abort(self, transaction):
        return self.app.tpc_abort(transaction=transaction)

    def tpc_finish(self, transaction, f=None):
        tid = self.app.tpc_finish(transaction=transaction,
            tryToResolveConflict=self.tryToResolveConflict, f=f)
        # XXX: Note that when undoing changes, the following is useless because
        #      a temporary Storage object is used to commit.
        #      See also testZODB.NEOZODBTests.checkMultipleUndoInOneTransaction
        if self._snapshot_tid:
            self._snapshot_tid = add64(tid, 1)
        return tid

    @check_read_only
    def store(self, oid, serial, data, version, transaction):
        assert version == '', 'Versions are not supported'
        return self.app.store(oid=oid, serial=serial,
            data=data, version=version, transaction=transaction)

    @check_read_only
    def deleteObject(self, oid, serial, transaction):
        self.app.store(oid=oid, serial=serial, data='', version=None,
            transaction=transaction)

    # mutliple revisions
    def loadSerial(self, oid, serial):
        try:
            return self.app.load(oid, serial)[0]
        except NEOStorageNotFoundError:
            raise POSException.POSKeyError(oid)

    def loadBefore(self, oid, tid):
        try:
            return self.app.load(oid, None, tid)
        except NEOStorageDoesNotExistError:
            raise POSException.POSKeyError(oid)
        except NEOStorageNotFoundError:
            return None

    def iterator(self, start=None, stop=None):
        # Iterator lives in its own transaction, so get a fresh snapshot.
        snapshot_tid = self.lastTransaction()
        if stop is None:
            stop = snapshot_tid
        else:
            stop = min(snapshot_tid, stop)
        return self.app.iterator(start, stop)

    # undo
    @check_read_only
    def undo(self, transaction_id, txn):
        return self.app.undo(self._snapshot_tid, undone_tid=transaction_id,
            txn=txn, tryToResolveConflict=self.tryToResolveConflict)


    @check_read_only
    def undoLog(self, first=0, last=-20, filter=None):
        return self.app.undoLog(first, last, filter)

    def supportsUndo(self):
        return True

    def supportsTransactionalUndo(self):
        return True

    @check_read_only
    def abortVersion(self, src, transaction):
        return self.app.abortVersion(src, transaction)

    @check_read_only
    def commitVersion(self, src, dest, transaction):
        return self.app.commitVersion(src, dest, transaction)

    def loadEx(self, oid, version):
        try:
            data, serial, _ = self.app.load(oid, None, self._snapshot_tid)
        except NEOStorageNotFoundError:
            raise POSException.POSKeyError(oid)
        return data, serial, ''

    def __len__(self):
        return self.app.getStorageSize()

    def registerDB(self, db, limit=None):
        self.app.registerDB(db, limit)

    def history(self, oid, version=None, size=1, filter=None):
        try:
            return self.app.history(oid, version, size, filter)
        except NEOStorageNotFoundError:
            raise KeyError

    def sync(self, force=True):
        # Increment by one, as we will use this as an excluded upper
        # bound (loadBefore).
        self._snapshot_tid = add64(self.lastTransaction(), 1)

    def copyTransactionsFrom(self, source, verbose=False):
        """ Zope compliant API """
        return self.app.importFrom(source, None, None,
                self.tryToResolveConflict)

    def importFrom(self, source, start=None, stop=None):
        """ Allow import only a part of the source storage """
        return self.app.importFrom(source, start, stop,
                self.tryToResolveConflict)

    def restore(self, oid, serial, data, version, prev_txn, transaction):
        raise NotImplementedError

    def pack(self, t, referencesf, gc=False):
        if gc:
            neo.lib.logging.warning(
                'Garbage Collection is not available in NEO, '
                'please use an external tool. Packing without GC.')
        self.app.pack(t)

    def lastSerial(self):
        # seems unused
        raise NotImplementedError

    def lastTransaction(self):
        # Used in ZODB unit tests
        return self.app.lastTransaction()

    def _clear_temp(self):
        raise NotImplementedError

    def set_max_oid(self, possible_new_max_oid):
        # seems used only by FileStorage
        raise NotImplementedError

    def cleanup(self):
        # Used in unit tests to remove local database files.
        # We have no such thing, so make this method a no-op.
        pass

    def close(self):
        self.app.close()

    def getTid(self, oid):
        try:
            return self.app.getLastTID(oid)
        except NEOStorageNotFoundError:
            raise KeyError

    def checkCurrentSerialInTransaction(self, oid, serial, transaction):
        self.app.checkCurrentSerialInTransaction(oid, serial, transaction)

    def new_instance(self):
        return Storage(*self._init_args, **self._init_kw)



#
# Copyright (C) 2009-2010  Nexedi SA
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

import unittest
from mock import Mock
from neo.tests import NeoUnitTestBase
from neo.lib.pt import PartitionTable
from neo.storage.app import Application
from neo.storage.handlers.initialization import InitializationHandler
from neo.lib.protocol import CellStates, ProtocolError
from neo.lib.exception import PrimaryFailure

class StorageInitializationHandlerTests(NeoUnitTestBase):

    def setUp(self):
        NeoUnitTestBase.setUp(self)
        self.prepareDatabase(number=1)
        # create an application object
        config = self.getStorageConfiguration(master_number=1)
        self.app = Application(config)
        self.verification = InitializationHandler(self.app)
        # define some variable to simulate client and storage node
        self.master_port = 10010
        self.storage_port = 10020
        self.client_port = 11011
        self.num_partitions = 1009
        self.num_replicas = 2
        self.app.operational = False
        self.app.load_lock_dict = {}
        self.app.pt = PartitionTable(self.num_partitions, self.num_replicas)

    def tearDown(self):
        self.app.close()
        del self.app
        super(StorageInitializationHandlerTests, self).tearDown()

    # Common methods
    def getLastUUID(self):
        return self.uuid

    def getClientConnection(self):
        address = ("127.0.0.1", self.client_port)
        return self.getFakeConnection(uuid=self.getNewUUID(), address=address)

    def test_03_connectionClosed(self):
        conn = self.getClientConnection()
        self.app.listening_conn = object() # mark as running
        self.assertRaises(PrimaryFailure, self.verification.connectionClosed, conn,)
        # nothing happens
        self.checkNoPacketSent(conn)

    def test_09_answerPartitionTable(self):
        # send a table
        conn = self.getClientConnection()
        self.app.pt = PartitionTable(3, 2)
        node_1 = self.getNewUUID()
        node_2 = self.getNewUUID()
        node_3 = self.getNewUUID()
        self.app.uuid = node_1
        # SN already know all nodes
        self.app.nm.createStorage(uuid=node_1)
        self.app.nm.createStorage(uuid=node_2)
        self.app.nm.createStorage(uuid=node_3)
        self.assertEqual(self.app.dm.getPartitionTable(), [])
        row_list = [(0, ((node_1, CellStates.UP_TO_DATE), (node_2, CellStates.UP_TO_DATE))),
                    (1, ((node_3, CellStates.UP_TO_DATE), (node_1, CellStates.UP_TO_DATE))),
                    (2, ((node_2, CellStates.UP_TO_DATE), (node_3, CellStates.UP_TO_DATE)))]
        self.assertFalse(self.app.pt.filled())
        # send a complete new table and ack
        self.verification.answerPartitionTable(conn, 2, row_list)
        self.assertTrue(self.app.pt.filled())
        self.assertEqual(self.app.pt.getID(), 2)
        self.assertNotEqual(self.app.dm.getPartitionTable(), [])


if __name__ == "__main__":
    unittest.main()




#
# Copyright (C) 2006-2010  Nexedi SA
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

import neo.lib.pt
from struct import pack, unpack
from neo.lib.protocol import CellStates
from neo.lib.pt import PartitionTableException
from neo.lib.pt import PartitionTable

class PartitionTable(PartitionTable):
    """This class manages a partition table for the primary master node"""

    def setID(self, id):
        assert isinstance(id, (int, long)) or id is None, id
        self._id = id

    def setNextID(self):
        if self._id is None:
            raise RuntimeError, 'I do not know the last Partition Table ID'
        self._id += 1
        return self._id

    def make(self, node_list):
        """Make a new partition table from scratch."""
        # start with the first PTID
        self._id = 1
        # First, filter the list of nodes.
        node_list = [n for n in node_list if n.isRunning() \
                and n.getUUID() is not None]
        if len(node_list) == 0:
            # Impossible.
            raise RuntimeError, 'cannot make a partition table with an ' \
                    'empty storage node list'

        # Take it into account that the number of storage nodes may be less
        # than the number of replicas.
        repeats = min(self.nr + 1, len(node_list))
        index = 0
        for offset in xrange(self.np):
            row = []
            for _ in xrange(repeats):
                node = node_list[index]
                row.append(neo.lib.pt.Cell(node))
                self.count_dict[node] = self.count_dict.get(node, 0) + 1
                index += 1
                if index == len(node_list):
                    index = 0
            self.partition_list[offset] = row

        self.num_filled_rows = self.np

    def findLeastUsedNode(self, excluded_node_list = ()):
        min_count = self.np + 1
        min_node = None
        for node, count in self.count_dict.iteritems():
            if min_count > count \
                    and node not in excluded_node_list \
                    and node.isRunning():
                min_node = node
                min_count = count
        return min_node

    def dropNode(self, node):
        cell_list = []
        uuid = node.getUUID()
        for offset, row in enumerate(self.partition_list):
            if row is None:
                continue
            for cell in row:
                if cell.getNode() is node:
                    if not cell.isFeeding():
                        # If this cell is not feeding, find another node
                        # to be added.
                        node_list = [c.getNode() for c in row]
                        n = self.findLeastUsedNode(node_list)
                        if n is not None:
                            row.append(neo.lib.pt.Cell(n,
                                    CellStates.OUT_OF_DATE))
                            self.count_dict[n] += 1
                            cell_list.append((offset, n.getUUID(),
                                              CellStates.OUT_OF_DATE))
                    row.remove(cell)
                    cell_list.append((offset, uuid, CellStates.DISCARDED))
                    break

        try:
            del self.count_dict[node]
        except KeyError:
            pass

        return cell_list

    def load(self, ptid, row_list, nm):
        """
        Load a partition table from a storage node during the recovery.
        Return the new storage nodes registered
        """
        # check offsets
        for offset, _row in row_list:
            if offset >= self.getPartitions():
                raise IndexError, offset
        # store the partition table
        self.clear()
        self._id = ptid
        new_nodes = []
        for offset, row in row_list:
            for uuid, state in row:
                node = nm.getByUUID(uuid)
                if node is None:
                    node = nm.createStorage(uuid=uuid)
                    new_nodes.append(node.asTuple())
                self.setCell(offset, node, state)
        return new_nodes

    def setUpToDate(self, node, offset):
        """Set a cell as up-to-date"""
        uuid = node.getUUID()
        # check the partition is assigned and known as outdated
        for cell in self.getCellList(offset):
            if cell.getUUID() == uuid:
                if not cell.isOutOfDate():
                    raise PartitionTableException('Non-oudated partition')
                break
        else:
            raise PartitionTableException('Non-assigned partition')

        # update the partition table
        cell_list = [self.setCell(offset, node, CellStates.UP_TO_DATE)]
        cell_list = [(offset, uuid, CellStates.UP_TO_DATE)]

        # If the partition contains a feeding cell, drop it now.
        for feeding_cell in self.getCellList(offset):
            if feeding_cell.isFeeding():
                cell_list.append(self.removeCell(offset,
                    feeding_cell.getNode()))
                break

        return cell_list

    def addNode(self, node):
        """Add a node. Take it into account that it might not be really a new
        node. The strategy is, if a row does not contain a good number of
        cells, add this node to the row, unless the node is already present
        in the same row. Otherwise, check if this node should replace another
        cell."""
        cell_list = []
        node_count = self.count_dict.get(node, 0)
        for offset, row in enumerate(self.partition_list):
            feeding_cell = None
            max_count = 0
            max_cell = None
            num_cells = 0
            skip = False
            for cell in row:
                if cell.getNode() == node:
                    skip = True
                    break
                if cell.isFeeding():
                    feeding_cell = cell
                else:
                    num_cells += 1
                    count = self.count_dict[cell.getNode()]
                    if count > max_count:
                        max_count = count
                        max_cell = cell

            if skip:
                continue

            if num_cells <= self.nr:
                row.append(neo.lib.pt.Cell(node, CellStates.OUT_OF_DATE))
                cell_list.append((offset, node.getUUID(),
                    CellStates.OUT_OF_DATE))
                node_count += 1
            else:
                if max_count - node_count > 1:
                    if feeding_cell is not None or max_cell.isOutOfDate():
                        # If there is a feeding cell already or it is
                        # out-of-date, just drop the node.
                        row.remove(max_cell)
                        cell_list.append((offset, max_cell.getUUID(),
                                          CellStates.DISCARDED))
                        self.count_dict[max_cell.getNode()] -= 1
                    else:
                        # Otherwise, use it as a feeding cell for safety.
                        max_cell.setState(CellStates.FEEDING)
                        cell_list.append((offset, max_cell.getUUID(),
                                          CellStates.FEEDING))
                        # Don't count a feeding cell.
                        self.count_dict[max_cell.getNode()] -= 1
                    row.append(neo.lib.pt.Cell(node, CellStates.OUT_OF_DATE))
                    cell_list.append((offset, node.getUUID(),
                                      CellStates.OUT_OF_DATE))
                    node_count += 1

        self.count_dict[node] = node_count
        self.log()
        return cell_list

    def tweak(self):
        """Test if nodes are distributed uniformly. Otherwise, correct the
        partition table."""
        changed_cell_list = []

        for offset, row in enumerate(self.partition_list):
            removed_cell_list = []
            feeding_cell = None
            out_of_date_cell_list = []
            up_to_date_cell_list = []
            for cell in row:
                if cell.getNode().isBroken():
                    # Remove a broken cell.
                    removed_cell_list.append(cell)
                elif cell.isFeeding():
                    if feeding_cell is None:
                        feeding_cell = cell
                    else:
                        # Remove an excessive feeding cell.
                        removed_cell_list.append(cell)
                elif cell.isOutOfDate():
                    out_of_date_cell_list.append(cell)
                else:
                    up_to_date_cell_list.append(cell)

            # If all cells are up-to-date, a feeding cell is not required.
            if len(out_of_date_cell_list) == 0 and feeding_cell is not None:
                removed_cell_list.append(feeding_cell)

            ideal_num = self.nr + 1
            while len(out_of_date_cell_list) + len(up_to_date_cell_list) > \
                    ideal_num:
                # This row contains too many cells.
                if len(up_to_date_cell_list) > 1:
                    # There are multiple up-to-date cells, so choose whatever
                    # used too much.
                    cell_list = out_of_date_cell_list + up_to_date_cell_list
                else:
                    # Drop an out-of-date cell.
                    cell_list = out_of_date_cell_list

                max_count = 0
                chosen_cell = None
                for cell in cell_list:
                    count = self.count_dict[cell.getNode()]
                    if max_count < count:
                        max_count = count
                        chosen_cell = cell
                removed_cell_list.append(chosen_cell)
                try:
                    out_of_date_cell_list.remove(chosen_cell)
                except ValueError:
                    up_to_date_cell_list.remove(chosen_cell)

            # Now remove cells really.
            for cell in removed_cell_list:
                row.remove(cell)
                if not cell.isFeeding():
                    self.count_dict[cell.getNode()] -= 1
                changed_cell_list.append((offset, cell.getUUID(),
                    CellStates.DISCARDED))

        # Add cells, if a row contains less than the number of replicas.
        for offset, row in enumerate(self.partition_list):
            num_cells = 0
            for cell in row:
                if not cell.isFeeding():
                    num_cells += 1
            while num_cells <= self.nr:
                node = self.findLeastUsedNode([cell.getNode() for cell in row])
                if node is None:
                    break
                row.append(neo.lib.pt.Cell(node, CellStates.OUT_OF_DATE))
                changed_cell_list.append((offset, node.getUUID(),
                    CellStates.OUT_OF_DATE))
                self.count_dict[node] += 1
                num_cells += 1

        self.log()
        return changed_cell_list

    def outdate(self):
        """Outdate all non-working nodes."""
        cell_list = []
        for offset, row in enumerate(self.partition_list):
            for cell in row:
                if not cell.getNode().isRunning() and not cell.isOutOfDate():
                    cell.setState(CellStates.OUT_OF_DATE)
                    cell_list.append((offset, cell.getUUID(),
                        CellStates.OUT_OF_DATE))
        return cell_list




#
# Copyright (C) 2006-2010  Nexedi SA
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from neo.lib.exception import DatabaseFailure
from neo.storage.database.manager import DatabaseManager

DATABASE_MANAGER_DICT = {}

try:
    from neo.storage.database.mysqldb import MySQLDatabaseManager
except ImportError:
    pass
else:
    DATABASE_MANAGER_DICT['MySQL'] = MySQLDatabaseManager

try:
    from neo.storage.database.btree import BTreeDatabaseManager
except ImportError:
    pass
else:
    # XXX: warning: name might change in the future.
    DATABASE_MANAGER_DICT['BTree'] = BTreeDatabaseManager

if not DATABASE_MANAGER_DICT:
    raise ImportError('No database back-end available.')

def buildDatabaseManager(name, config):
    if name is None:
        name = DATABASE_MANAGER_DICT.keys()[0]
    adapter_klass = DATABASE_MANAGER_DICT.get(name, None)
    if adapter_klass is None:
        raise DatabaseFailure('Cannot find a database adapter <%s>' % name)
    return adapter_klass(config)

